/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include <string>

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "Components/SceneComponent.h"
#include "Components/StaticMeshComponent.h"
#include "Components/DrawFrustumComponent.h"
#include "Engine/TextureRenderTarget2D.h"
#include "Engine/SceneCapture2D.h"
#include "TimerManager.h"

#include "ExpPipelineTypes.h"
#include "ForegroundCutoffMeshComponent.h"
#include "MixCastSliceCanvas.h"
#include "MixCastOpaqueCanvas.h"
#include "MixCastSceneCaptureComponent2D.h"
#include "OpaqueProcessingCaptureComponent.h"
#include "Transmission/Generated/VirtualCamera_types.h"
#include "Transmission/Unreal-Interop/SharedQueue/SharedQueueConsumer.hpp"

#include "ExpCamera.generated.h"

using namespace mixcast::data;
using namespace mixcast::thrift;

class FMixCast;
class FExpFrameSender;

UCLASS(HideCategories = ("Actor", "Tick", "HLOD", "Input", "Replication"))
class MIXCAST_API AExpCamera : public AActor
{
	GENERATED_UCLASS_BODY()

public:
	static void DisposeAllCachedQueues();

	std::string Identifier;

	DECLARE_EVENT_OneParam(AExpCamera, FFrameStartedEvent, AExpCamera*)
	static FFrameStartedEvent& OnFrameStarted() { return FrameStarted; }
	DECLARE_EVENT_OneParam(AExpCamera, FFrameEndedEvent, AExpCamera*)
	static FFrameEndedEvent& OnFrameEnded() { return FrameEnded; }

	void SetActorHidden(AActor* actor);

	void CaptureIfNeeded();

	float GetNearClipPlane();
	ExpFrame GetLatestFrameInfo() { return latestFrameInfo; }

	UMixCastSliceCanvas* GetSliceCanvas() { return LayerCanvas; }

#if MIXCAST_SHADER_PASSES
	UTextureRenderTarget2D* GetHdrOpaqueLayerTarget() { return OpaqueTarget; }
	UMixCastOpaqueCanvas* GetOpaqueCanvas() { return OpaqueCanvas; }
#endif
protected:
	static FFrameStartedEvent FrameStarted;
	static FFrameEndedEvent FrameEnded;

	static TMap<FString, SharedQueueConsumer<ExpFrame>*> cachedFrameQueues;

	FMixCast* sdkModule;

	UPROPERTY()
		USceneComponent* CameraTransform;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Full Capture")
		UMixCastSceneCaptureComponent2D* FullCapture;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Foreground Capture")
		UMixCastSceneCaptureComponent2D* ForegroundCapture;
	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category = "Foreground Capture")
		UForegroundCutoffMeshComponent* ForegroundCutoffMesh;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Textures")
		UMixCastSliceCanvas* LayerCanvas;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Textures")
		UTextureRenderTarget2D* OpaqueTarget;


	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Opaque Processing")
		UOpaqueProcessingCaptureComponent* OpaqueProcessingCapture;
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Textures")
		UTextureRenderTarget2D* ProcessedOpaqueColorTarget;
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Textures")
		UMixCastOpaqueCanvas* OpaqueCanvas;


	void BeginPlay() override;
	void EndPlay(EEndPlayReason::Type EndPlayReason) override;

	void CaptureFrame();

	void UpdateTransform(FVector camPos, FQuat camRot);
	void UpdateCameraParameters();
	void ResetHiddenActors();

	void TriggerSceneCaptures();
	void AtlasSlices();

private:
	SharedQueueConsumer<ExpFrame>* requestedFrames;

	ExpFrame lastReceivedFrame;	//could be a future prediction
	bool receivedLastFrameTooEarly;

	ExpFrame latestFrameInfo;	//the latest in the past/present
	unsigned int renderedFrameCount;
	FExpFrameSender* framePipe;

	bool fullRenderSetAsTransparent;

	int rendersLeft;
	FDelegateHandle onPreFullRender, onPostFullRender, onPreForegroundRender, onPostForegroundRender;

	void HandleSceneCaptureStarted(UMixCastSceneCaptureComponent2D* sceneCapture);
	void HandleSceneCaptureEnded(UMixCastSceneCaptureComponent2D* sceneCapture);

	UMixCastSceneCaptureComponent2D* CreateCaptureComponent(bool isForeground);

	void CreateSliceCanvas(unsigned int width, unsigned int height, bool separateAlpha);
	void DestroySliceCanvas();
#if MIXCAST_SHADER_PASSES
	void CreateOpaqueLayerTarget(unsigned int width, unsigned int height);
	void DestroyOpaqueLayerTarget();
#endif
};
