/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "MixCastActorVisibility.h"
#include "MixCastSdkActor.h"
#include "MixCast.h"
#include "Data/SdkData.h"
#include "Capture/ExpPipelineTypes.h"
#include "Capture/ExpCamera.h"
#include "Transmission/Generated/VirtualCamera_types.h"

#include "Engine/Engine.h"
#include "GameFramework/PlayerController.h"

UMixCastActorVisibility::UMixCastActorVisibility()
{

}

void UMixCastActorVisibility::BeginPlay()
{
	Super::BeginPlay();

	if (!this->showForRegularCameras) {
		APlayerController* player = GEngine->GetFirstLocalPlayerController(GetWorld());
		if (player)
			player->HiddenActors.Add(GetOwner());
	}

	FrameStartedDelegate = AExpCamera::OnFrameStarted().AddUObject(this, &UMixCastActorVisibility::HandleFrameStarted);
}
void UMixCastActorVisibility::EndPlay(const EEndPlayReason::Type endPlayReason)
{
	Super::EndPlay(endPlayReason);

	if (!this->showForRegularCameras) {
		APlayerController* player = GEngine->GetFirstLocalPlayerController(GetWorld());
		if (player)
			player->HiddenActors.Remove(GetOwner());
	}

	AExpCamera::OnFrameStarted().Remove(FrameStartedDelegate);
}

void UMixCastActorVisibility::HandleFrameStarted(AExpCamera* cam)
{
	if (!ShouldShowToCamera(cam->GetLatestFrameInfo()))
		cam->SetActorHidden(GetOwner());
}

bool UMixCastActorVisibility::ShouldShowToCamera(const ExpFrame& frameInfo)
{
	bool showForThisMixCastCamera = showForMixCastCameras;
	switch (mixCastCameraMode.GetValue())
	{
	case EMixCastCameraMode::Always:
		break;
	case EMixCastCameraMode::IfAnyConditionMet:
		if (!HasAnyFlag(frameInfo))
			showForThisMixCastCamera = !showForThisMixCastCamera;
		break;
	case EMixCastCameraMode::IfAllConditionsMet:
		if (!HasAllFlags(frameInfo))
			showForThisMixCastCamera = !showForThisMixCastCamera;
		break;
	}
	return showForThisMixCastCamera;
}
bool UMixCastActorVisibility::HasAnyFlag(const ExpFrame& frameInfo)
{
	switch (renderTypeCondition.GetValue())
	{
	case ERenderTypeCondition::Mixed:
		if (frameInfo.renderForeground)
			return true;
		break;
	case ERenderTypeCondition::Virtual:
		if (!frameInfo.renderForeground)
			return true;
		break;
	}

	switch (backdropStyleCondition.GetValue())
	{
	case EBackdropStyleCondition::Virtual:
		if (!frameInfo.HasCamFlag(ExpCamFlagBit::Translucent))
			return true;
		break;
	case EBackdropStyleCondition::Physical:
		if (frameInfo.HasCamFlag(ExpCamFlagBit::Translucent))
			return true;
		break;
	}

	switch (perspectiveCondition.GetValue())
	{
	case EPerspectiveCondition::First:
		if (frameInfo.HasCamFlag(ExpCamFlagBit::FirstPerson))
			return true;
		break;
	case EPerspectiveCondition::Third:
		if (!frameInfo.HasCamFlag(ExpCamFlagBit::FirstPerson))
			return true;
		break;
	}

	return false;
}
bool UMixCastActorVisibility::HasAllFlags(const ExpFrame& frameInfo)
{
	switch (renderTypeCondition.GetValue())
	{
	case ERenderTypeCondition::Mixed:
		if (!frameInfo.renderForeground)
			return false;
		break;
	case ERenderTypeCondition::Virtual:
		if (frameInfo.renderForeground)
			return false;
		break;
	}

	switch (backdropStyleCondition.GetValue())
	{
	case EBackdropStyleCondition::Virtual:
		if (frameInfo.HasCamFlag(ExpCamFlagBit::Translucent))
			return false;
		break;
	case EBackdropStyleCondition::Physical:
		if (!frameInfo.HasCamFlag(ExpCamFlagBit::Translucent))
			return false;
		break;
	}

	switch (perspectiveCondition.GetValue())
	{
	case EPerspectiveCondition::First:
		if (!frameInfo.HasCamFlag(ExpCamFlagBit::FirstPerson))
			return false;
		break;
	case EPerspectiveCondition::Third:
		if (frameInfo.HasCamFlag(ExpCamFlagBit::FirstPerson))
			return false;
		break;
	}

	return true;
}