/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "OpaqueProcessingCaptureComponent.h"
#if MIXCAST_SHADER_PASSES
#include "ExpCamera.h"
#include "Data/SdkData.h"
#include "MixCast.h"

#include "EngineModule.h"
#include "EngineGlobals.h"
#include "RenderingThread.h"
#include "SceneInterface.h"
#include "SceneView.h"
#include "Engine/TextureRenderTarget2D.h"
#include "Runtime/Renderer/Private/PostProcess/SceneRenderTargets.h"
#include "ScreenRendering.h"

void UOpaqueProcessingCaptureComponent::Initialize()
{
	bCaptureEveryFrame = false;
	bCaptureOnMovement = false;
	bAlwaysPersistRenderingState = true;
	PrimitiveRenderMode = ESceneCapturePrimitiveRenderMode::PRM_UseShowOnlyList;

#if UE_4_25_OR_LATER
	bUseRayTracingIfEnabled = false;
#endif
}

void UOpaqueProcessingCaptureComponent::BeginPlay()
{
	reinsertOpaqueDel.BindUObject(this, &UOpaqueProcessingCaptureComponent::ReinsertOpaque_RenderThread);

	dummyRenderer = NewObject<UStaticMeshComponent>(this);
	dummyRenderer->RegisterComponent();
	dummyRenderer->AttachTo(this);
	dummyRenderer->SetOnlyOwnerSee(true);
	ShowOnlyComponents.Add(dummyRenderer);

	Super::BeginPlay();
}
void UOpaqueProcessingCaptureComponent::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
	Super::EndPlay(EndPlayReason);

	ShowOnlyComponents.Remove(dummyRenderer);
	dummyRenderer->DestroyComponent();
	dummyRenderer = NULL;

	reinsertOpaqueDel.Unbind();
}

void UOpaqueProcessingCaptureComponent::UpdateSceneCaptureContents(FSceneInterface* Scene)
{
	if (OpaqueTarget == NULL || !OpaqueTarget->IsValidLowLevel())
		return;
	
	ENQUEUE_RENDER_COMMAND(PreMixCastOpaqueProcessCommand)(
		[this](FRHICommandListImmediate& /*RHICmdList*/)
		{
			auto& renderer = GetRendererModule();
#if UE_4_26_OR_LATER
			reinsertOpaqueDelHandle = renderer.RegisterPostOpaqueRenderDelegate(reinsertOpaqueDel);
#else
			renderer.RegisterPostOpaqueRenderDelegate(reinsertOpaqueDel);
#endif
		}
	);

	Super::UpdateSceneCaptureContents(Scene);

	ENQUEUE_RENDER_COMMAND(PostMixCastOpaqueProcessCommand)(
		[this](FRHICommandListImmediate& /*RHICmdList*/)
		{
			auto& renderer = GetRendererModule();
#if UE_4_26_OR_LATER
			renderer.RemovePostOpaqueRenderDelegate(reinsertOpaqueDelHandle);
#else
			renderer.RegisterPostOpaqueRenderDelegate(NULL);
#endif
		}
	);
}

void UOpaqueProcessingCaptureComponent::ReinsertOpaque_RenderThread(FPostOpaqueRenderParameters& Parameters)
{
	if (OpaqueTarget != NULL && OpaqueTarget->IsValidLowLevel())
		FMixCastShaders::RewriteOpaque(*Parameters.RHICmdList, *(FSceneView*)Parameters.Uid, OpaqueTarget->Resource->TextureRHI);
}
#endif