/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include <chrono>
#include <string>

#include "CoreMinimal.h"
#include "Containers/UnrealString.h"
#include "Misc/App.h"
#include "Misc/ConfigCacheIni.h"
#include "Engine/Engine.h"
#include "Engine/GameViewportClient.h"
#include "Widgets/SWindow.h"
#include "Windows/WindowsPlatformProcess.h"
#include "Windows/WindowsPlatformMisc.h"
#include "HardwareInfo.h"

#include "MixCastDefines.h"
#include "MixCast/Public/IMixCast.h"
#include "MixCast/Private/Transmission/Generated/ExperienceMetadata_types.h"

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif

class MIXCAST_API ProjectUtils
{
public:
	static FString GetEngineVersion()
	{
		return FString::Printf(TEXT("%d.%d.%d"), ENGINE_MAJOR_VERSION, ENGINE_MINOR_VERSION, ENGINE_PATCH_VERSION);
	}

	static FString GetCompanyName()
	{
		FString CompanyName;
		GConfig->GetString(TEXT("/Script/EngineSettings.GeneralProjectSettings"), TEXT("CompanyName"), CompanyName, GGameIni);

		if (CompanyName.Len() == 0)
		{
			CompanyName = FString(TEXT("DefaultCompany"));
		}

		return CompanyName;
	}

	static FString GetDeviceId()
	{
		PRAGMA_DISABLE_DEPRECATION_WARNINGS
		FString DeviceId = FPlatformMisc::GetMacAddressString();
		PRAGMA_ENABLE_DEPRECATION_WARNINGS

		// The device ID uses the MAC address, which is not guaranteed to be supported on all platforms.
		if (DeviceId.Len() == 0)
		{
			DeviceId = FString(TEXT("NoDeviceId"));
		}

		return DeviceId;
	}

	static FString GetExecutablePath()
	{
		if (FApp::IsGame())
		{
#if UE_4_22_OR_LATER
			return FString(FWindowsPlatformProcess::ExecutablePath());
#else
			return FWindowsPlatformProcess::GetApplicationName(FWindowsPlatformProcess::GetCurrentProcessId());
#endif
		}
		else
		{
			return FPaths::Combine(FPlatformProcess::BaseDir(), FPlatformProcess::ExecutableName(false));
		}
	}

	static uint64 GetMainWindowHandle()
	{
		if (!GEngine || !GEngine->GameViewport || !GEngine->GameViewport->GetWindow().IsValid() || !GEngine->GameViewport->GetWindow()->GetNativeWindow().IsValid())
			return 0;
		const void* hwnd = GEngine->GameViewport->GetWindow()->GetNativeWindow()->GetOSWindowHandle();
		return (uint64)hwnd;
	}

	static FString GetProjectName()
	{
#if UE_4_18_OR_LATER
		return FString(FApp::GetProjectName());
#else
		return FString(FApp::GetGameName());
#endif
	}

	static FString GetProjectID()
	{
		FString ProjectID;
		GConfig->GetString(TEXT("/Script/EngineSettings.GeneralProjectSettings"), TEXT("ProjectID"), ProjectID, GGameIni);

		if (ProjectID.Len() == 0)
		{
			ProjectID = FString(TEXT("---"));
		}

		return ProjectID;
	}

	static mixcast::data::ExpGraphicsType::type GetGraphicsType()
	{
		FString rhiId = FHardwareInfo::GetHardwareInfo(NAME_RHI);
		if (rhiId.Equals(FString(TEXT("D3D12"))))
			return mixcast::data::ExpGraphicsType::DX12;
		else if (rhiId.Equals(FString(TEXT("D3D11"))))
			return mixcast::data::ExpGraphicsType::DX11;
		else
			return mixcast::data::ExpGraphicsType::UNKNOWN;
	}
};

#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif